/*
 *	Sherlock Indexer -- Helper Functions for Data Structure Access
 *
 *	(c) 2004-2005 Martin Mares <mj@ucw.cz>
 */

#include "sherlock/sherlock.h"
#include "indexer/indexer.h"

/* Keeping card count */

uns card_count = ~0U;
uns new_count = ~0U;

void
set_card_count(uns cc)
{
  if (card_count == ~0U)
    card_count = cc;
  else
    ASSERT(card_count == cc);
}

void
set_new_count(uns nc)
{
  if (new_count == ~0U)
    new_count = nc;
  else
    ASSERT(new_count == nc);
}

/* Dealing with notes */

struct partmap *notes_partmap, *notes_new_partmap;

void
notes_part_map(uns rw)
{
  ASSERT(!notes_partmap);
  notes_partmap = partmap_open(index_name(fn_notes), rw);
  ASSERT(!(partmap_size(notes_partmap) % sizeof(struct card_note)));
  set_card_count(partmap_size(notes_partmap) / sizeof(struct card_note));
}

void
notes_part_unmap(void)
{
  ASSERT(notes_partmap);
  partmap_close(notes_partmap);
  notes_partmap = NULL;
}

void
notes_new_part_map(uns rw)
{
  ASSERT(!notes_new_partmap);
  notes_new_partmap = partmap_open(index_name(fn_notes_new), rw);
  ASSERT(!(partmap_size(notes_new_partmap) % sizeof(struct card_note)));
  set_new_count(partmap_size(notes_new_partmap) / sizeof(struct card_note));
}

void
notes_new_part_unmap(void)
{
  ASSERT(notes_new_partmap);
  partmap_close(notes_new_partmap);
  notes_new_partmap = NULL;
}

/* Dealing with card attributes */

struct card_attr *attrs;
struct partmap *attrs_partmap;

void
attrs_map(uns rw)
{
  uns size;
  ASSERT(!attrs);
  attrs = mmap_file(index_name(fn_attributes), &size, rw);
  ASSERT(!(size % sizeof(struct card_attr)));
  set_card_count(size / sizeof(struct card_attr));
}

void
attrs_unmap(void)
{
  ASSERT(attrs);
  munmap_file(attrs, card_count * sizeof(struct card_attr));
  attrs = NULL;
}

void
attrs_part_map(uns rw)
{
  ASSERT(!attrs_partmap);
  attrs_partmap = partmap_open(index_name(fn_attributes), rw);
  ASSERT(!(partmap_size(attrs_partmap) % sizeof(struct card_attr)));
  set_card_count(partmap_size(attrs_partmap) / sizeof(struct card_attr));
}

void
attrs_part_unmap(void)
{
  ASSERT(attrs_partmap);
  partmap_close(attrs_partmap);
  attrs_partmap = NULL;
}
