/*
 *	Sherlock Indexer -- Fetching of Cards for Second Indexer Pass
 *
 *	(c) 2001--2005 Martin Mares <mj@ucw.cz>
 *	(c) 2003--2005 Robert Spalek <robert@ucw.cz>
 */

#include "sherlock/sherlock.h"
#include "lib/fastbuf.h"
#include "lib/mempool.h"
#include "lib/url.h"
#include "sherlock/object.h"
#include "sherlock/objread.h"
#include "indexer/indexer.h"

#include <string.h>
#include <fcntl.h>

uns fetch_id;

void
fetch_cards(void (*got_card)(struct card_attr *attr, struct odes *obj, struct card_note *note))
{
  struct mempool *pool;
  struct fastbuf *fb_labels, *fb_attrs, *fb_notes;
  struct odes *o;
  u32 next_label_id;
  struct card_attr attr;
  struct card_note note;
  byte buf[MAX_ATTR_SIZE];
  struct odes *main_hdr, *url_hdr, *current_hdr;
  struct obj_read_state read_state;
  struct bucket_source *src;

  pool = mp_new(16384);
  src = get_buck_init();
  fb_labels = index_bopen(fn_labels, O_RDONLY);
  fb_attrs = index_bopen(fn_attributes, O_RDONLY);
  fb_notes = index_bopen(fn_notes, O_RDONLY);
  fetch_id = -1;
  next_label_id = bgetl(fb_labels);
  while (breadb(fb_attrs, &attr, sizeof(attr)))
    {
      fetch_id++;
      breadb(fb_notes, &note, sizeof(note));
      if (attr.flags & (CARD_FLAG_EMPTY | CARD_FLAG_DUP))
	{
	  while (next_label_id == fetch_id)
	    {
	      bgetl(fb_labels);
	      bgetl(fb_labels);
	      uns cnt = bgetl(fb_labels);
	      /* This really can happen, for example when we attach frame backlinks to empty frames */
	      DBG("Ignoring label for skipped object %08x", fetch_id);
	      bskip(fb_labels, cnt);
	      next_label_id = bgetl(fb_labels);
	    }
	  continue;
	}
      mp_flush(pool);
      if (!src->get_next(src, pool, attr.card))
	die("Object pool changed unexpectedly");
      o = src->o;

      /* Pick all per-URL attributes and move them to the main header */
      main_hdr = obj_new(pool);
      struct oattr **Oa = &o->attrs, *oa;
      while (oa = *Oa)
	if (oa->attr == 'U' || strchr(label_attrs, oa->attr))
	  {
	    *Oa = oa->next;				// delete from this chain
	    oa->next = main_hdr->attrs;			// prepend to main_hdr, no need to check the existence
	    main_hdr->attrs = oa;
	  }
	else
	  Oa = &oa->next;
      if (attr.flags & CARD_FLAG_MERGED)		// merged cards get all per-URL attributes from labels
	main_hdr = obj_new(pool);
      obj_add_son_ref(o, 'U' + OBJ_ATTR_SON, main_hdr);
      current_hdr = url_hdr = main_hdr;

      /* Read all labels and create the corresponding headers */
      u32 src_card = 0;
      u32 src_redir = 0;
      while (next_label_id == fetch_id)
	{
	  u32 scard = bgetl(fb_labels);
	  u32 sredir = bgetl(fb_labels);
	  if (scard != src_card)
	    {
	      if (src_card)
		current_hdr = url_hdr = obj_add_son(o, 'U' + OBJ_ATTR_SON);
	      src_card = scard;
	      src_redir = scard;
	    }
	  if (sredir != src_redir)
	    {
	      current_hdr = obj_add_son(url_hdr, 'y' + OBJ_ATTR_SON);
	      src_redir = sredir;
	    }
	  uns cnt = bgetl(fb_labels);
	  sh_off_t stop = btell(fb_labels) + cnt;
	  obj_read_start(&read_state, current_hdr);
	  while (btell(fb_labels) < stop)
	    {
	      if (!bgets0(fb_labels, buf, sizeof(buf)))
		die("Unexpected inconsistency of label file");
	      if (buf[0] == 'x' && read_state.obj == current_hdr)
		obj_add_attr(o, buf[0], buf+1);
	      else
		obj_read_attr(&read_state, buf[0], buf+1);
	    }
	  obj_read_end(&read_state);
	  next_label_id = bgetl(fb_labels);
	}
      if (note.useful_size)
	obj_set_attr_num(o, 'k', note.useful_size);
      if (attr.flags & CARD_FLAG_FRAMESET)
	{
	  /* Frameset to be deleted, but CARD_FLAG_EMPTY is no longer set,
	   * which means that some labels have been attached, so we index it,
	   * but without the X attribute.
	   */
	  obj_set_attr(o, 'X', NULL);
	}
      got_card(&attr, o, &note);
    }
  ASSERT(next_label_id == ~0U);
  src->cleanup(src);
  bclose(fb_attrs);
  bclose(fb_labels);
  mp_delete(pool);
  log(L_INFO, "Processed %d objects", fetch_id+1);
}
