/*
 *	Sherlock Language Processing Library -- Stemming
 *
 *	(c) 2003--2005 Martin Mares <mj@ucw.cz>
 */

#include "lib/lib.h"
#include "lib/mempool.h"
#include "lang/lang.h"
#include "lang/stemmer.h"

#include <string.h>

static clist *(*stemmer_funcs[])(struct stemmer *, struct word_node *, struct mempool *) = {
#define STEMMER(id, name) [id] = stem_##name,
#include "lang/stemmers.h"
#undef STEMMER
};

static void (*stemmer_init_funcs[])(struct stemmer *) = {
#define STEMMER(id, name) [id] = stem_init_##name,
#include "lang/stemmers.h"
#undef STEMMER
};

byte *word_form_names[] = { "OTHER", "BASIC", "STEM", "LEMMA" };

clist *
lang_stem(struct stemmer *st, struct word_node *w, struct mempool *mp)
{
  ASSERT(st->id < ARRAY_SIZE(stemmer_funcs));
  return stemmer_funcs[st->id](st, w, mp);
}

void
lang_init_stemmers(void)
{
  struct stemmer *st;
  CLIST_WALK(st, stemmer_list)
    stemmer_init_funcs[st->id](st);
}

struct word_node *
word_list_add(struct mempool *mp, clist *l, byte *w, uns word_form, uns stem_form, uns variant, uns unaccented)
{
  int len = strlen(w);
  if (len > MAX_WORD_LEN)
    return NULL;
  struct word_node *n = mp_alloc(mp, sizeof(*n) + len + 1);
  n->w = (byte *)(n+1);
  memcpy(n->w, w, len+1);
  n->word_form = word_form;
  n->stem_form = stem_form;
  n->variant = variant;
  n->unaccented = unaccented;
  clist_add_tail(l, &n->n);
  return n;
}

struct word_node *
word_list_find(clist *l, byte *w, uns word_form, uns stem_form, uns variant, uns unaccented)
{
  struct word_node *n;
  CLIST_WALK(n, *l)
    if (!strcmp(n->w, w) &&
	(n->word_form == word_form || word_form == ~0U) &&
	(n->stem_form == stem_form || stem_form == ~0U) &&
	n->variant == variant || variant == ~0U &&
	n->unaccented == unaccented || unaccented == ~0U)
      return n;
  return NULL;
}

struct word_node *
word_list_add_unique(struct mempool *mp, clist *l, byte *w, uns word_form, uns stem_form, uns variant, uns unaccented)
{
  struct word_node *n = word_list_find(l, w, word_form, stem_form, variant, unaccented);
  if (n)
    return n;
  else
    return word_list_add(mp, l, w, word_form, stem_form, variant, unaccented);
}
